//+-------------------------------------------------------------------+
//|                                                    Bandswidth.mq4 |
//|                                         by Linuxser for Forex TSD |
//|                                                                   |
//| John Bollinger original formula is:                               |
//| (Upper BB - Lower BB)/middle BB                                   |
//+-------------------------------------------------------------------+
//------------------------------------------------------------------
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  clrLimeGreen
#property indicator_color2  clrLightGreen
#property indicator_color3  clrGray
#property indicator_width1  2
#property strict

//
//
//

enum enTimeFrames
{
         tf_cu  = 0,                                           // Current time frame
         tf_m1  = PERIOD_M1,                                   // 1 minute
         tf_m5  = PERIOD_M5,                                   // 5 minutes
         tf_m15 = PERIOD_M15,                                  // 15 minutes
         tf_m30 = PERIOD_M30,                                  // 30 minutes
         tf_h1  = PERIOD_H1,                                   // 1 hour
         tf_h4  = PERIOD_H4,                                   // 4 hours
         tf_d1  = PERIOD_D1,                                   // Daily
         tf_w1  = PERIOD_W1,                                   // Weekly
         tf_mn1 = PERIOD_MN1,                                  // Monthly
         tf_n1  = -1,                                          // First higher time frame
         tf_n2  = -2,                                          // Second higher time frame
         tf_n3  = -3,                                          // Third higher time frame
      };
input enTimeFrames    inpTimeFrame     = tf_cu;                // Timeframe to use
input int             BBPeriod         = 20;                   // Bollinger band period
input int             StdDeviation     = 2;                    // Standard deviation
input bool            alertsOn         = true;                 // Alerts?
input bool            alertsOnCurrent  = false;                // Alerts open bar?
input bool            alertsMessage    = true;                 // Alerts message?
input bool            alertsSound      = false;                // Alerts sound?
input bool            alertsNotify     = false;                // Alerts notification?
input bool            alertsEmail      = false;                // Alerts email?
input string          soundFile        = "alert2.wav";         // Alerts sound file
enum  enIterpolate
      {
         interolate_yes = (int)true,                               // Interpolate data when in multi time frame
         interolate_no  = int(false)                               // Do not interpolate data when in multi time frame
      };
input enIterpolate     inpInterpolate   = interolate_yes; 

double huu[],hdd[],val[],valc[],cnt[];
struct sGlobalStruct
{
   string indicatorFileName;
   int    indicatorTimeFrame;
};
sGlobalStruct global;
#define _mtfCall(_buff,_ind) iCustom(_Symbol,global.indicatorTimeFrame,global.indicatorFileName,tf_cu,BBPeriod,StdDeviation,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,_buff,_ind)
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,huu,INDICATOR_DATA); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,hdd,INDICATOR_DATA); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,val,INDICATOR_DATA); SetIndexStyle(2,DRAW_LINE);  SetIndexLabel(2,"Bandswidth("+(string)BBPeriod+","+(string)StdDeviation+")");
   SetIndexBuffer(3,valc,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,cnt, INDICATOR_CALCULATIONS);
   
   global.indicatorFileName  = WindowExpertName();
   global.indicatorTimeFrame = (enTimeFrames)timeFrameValue(inpTimeFrame);
   
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(global.indicatorTimeFrame)+" Bandswidth("+(string)BBPeriod+","+(string)StdDeviation+")");
return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   int i,limit=fmin(rates_total-prev_calculated+1,rates_total-1); cnt[0] = limit;
      if (global.indicatorTimeFrame!=_Period)
      {
        limit = (int)fmax(limit,fmin(rates_total-1,_mtfCall(4,0)*global.indicatorTimeFrame/_Period));
        for (i=limit; i>=0; i--)
        {
           int y = iBarShift(_Symbol,global.indicatorTimeFrame,time[i]);
              huu[i] = _mtfCall(0,y);
              hdd[i] = _mtfCall(1,y);
              val[i] = _mtfCall(2,y);
              
              //
              //
              //
                  
              if (!inpInterpolate || (i>0 && y==iBarShift(_Symbol,global.indicatorTimeFrame,time[i-1]))) continue;
              #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
              int n,k; datetime btime = iTime(NULL,global.indicatorTimeFrame,y);
                 for(n = 1; (i+n)<rates_total && time[i+n] >= btime; n++) continue;	
                 for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++) 
                 {
                    _interpolate(val); 
                    huu[i+k] = (huu[i]!= EMPTY_VALUE) ? _interpolate(val) : EMPTY_VALUE; 
  	                 hdd[i+k] = (hdd[i]!= EMPTY_VALUE) ? _interpolate(val) : EMPTY_VALUE; 
  	              }                    
       } 
   return(rates_total);
   } 
   
   //
   //
   //
            
   for (i=limit;i>=0;i--)
   { 
      double ub = iBands(NULL,0,BBPeriod,StdDeviation,0,PRICE_CLOSE,MODE_UPPER,i); 
      double lb = iBands(NULL,0,BBPeriod,StdDeviation,0,PRICE_CLOSE,MODE_LOWER,i);
      double cl = iMA(NULL,0,BBPeriod,0,MODE_SMA,PRICE_CLOSE,i);
         val[i]  = (ub!=lb) ? (ub-lb)/cl : 0;
         valc[i] = (i<rates_total-1) ? (val[i]>val[i+1]) ? 1 : (val[i]<val[i+1]) ? -1 : valc[i+1] : 0;  
         hdd[i]  = (valc[i]==-1) ? val[i] : EMPTY_VALUE;
         huu[i]  = (valc[i]== 1) ? val[i] : EMPTY_VALUE;  
   }
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (valc[whichBar] != valc[whichBar+1])
      {
         if (valc[whichBar] == 1)  doAlert(whichBar," sloping up");
         if (valc[whichBar] ==-1)  doAlert(whichBar," sloping down");
      }         
   }
return(rates_total);
}

//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //

          message = timeFrameToString(_Period)+" "+_Symbol+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Bollinger Bandwidth "+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(_Symbol+" Bollinger Bandwidth ",message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : fabs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)fmin(i+add,size-1)]);
}




       